------------------------------------------------------------------------
-- LINDA Module
------------------------------------------------------------------------
-- Aircraft: AeroSoft DHC6 Twin Otter Extended
--  Version: 1.3
--  Date: August 2014
--  Author: Guenter Steiner, Mark Hurst
------------------------------------------------------------------------

-- ## System functions ##

function InitVars ()

    -- uncomment to disable display
    -- AutopilotDisplayBlocked ()
    ipc.set("ACId", 1)
    ipc.sleep(30)
    AircraftIdent ()

end 

function Twotter_Switchsound_1 ()
    ipc.writeLvar("L:Twot_SwitchSound", 1)
end

function Twotter_Switchsound_2 ()
    ipc.writeLvar("L:Twot_Butclick", 1)
end

function AircraftIdent ()
    acftname = ipc.readSTR("3D00", 35)

    if string.find(acftname,"300",0,true)
    then ipc.set("ACId", 300)
    elseif string.find(acftname,"100",0,true)
    then ipc.set("ACId", 100)
    end
    DspShow ("ACId", ipc.get("ACId"))
end

function Timer ()

    TWalt = ipc.readLvar("Alt_alert_setting")/100
    DspALT(TWalt)



    -- Flaps
     FlapVar = ipc.readLvar("Flaps_Indicator_off_val")
     if FlapVar < 5 then FlapTxt = "up"                       FlapIdx = 0
     elseif FlapVar > 25 and FlapVar < 28 then FlapTxt = "10" FlapIdx = 10
     elseif FlapVar > 50 and FlapVar < 55 then FlapTxt = "20" FlapIdx = 20
     elseif FlapVar > 78 and FlapVar < 81 then FlapTxt = "30" FlapIdx = 30
     elseif FlapVar > 98 then FlapTxt = "dn"                  FlapIdx = 40
     end

    -- Flaplever

    FlpLvr = ipc.readUB("0BFC")*10
    if FlpLvr == 0 then FlpLvrTxt = "up"
    elseif FlpLvr == 1 then FlpLvrTxt = "10"
    elseif FlpLvr == 2 then FlpLvrTxt = "20"
    elseif FlpLvr == 3 then FlpLvrTxt = "30"
    elseif FlpLvr == 4 then FlpLvrTxt = "40"
    end

    -- Flap restrictions

    ACType = ipc.get("ACId")
    AirSpd = round(ipc.readUD(0x02BC) / 128)
    OnGround = ipc.readUW("0366")  -- plane on ground?

    if OnGround == 0 then   -- plane on ground?
        if ACType == 300 then
            if FlapIdx > 0 and AirSpd > 102 then
            FlapResTxt = "!"
            elseif FlapIdx > 20 and AirSpd > 95 then
            FlapResTxt = "!"
            elseif FlapIdx == 0 and AirSpd <= 102 then
            FlapResTxt = "."
            elseif FlapIdx > 0 and FlapIdx <= 20 and AirSpd <= 95 then
            FlapResTxt = "."

            else FlapResTxt = " "
            end
         end

         if ACType == 100 then
            if FlapIdx > 0 and AirSpd > 100 then
            FlapResTxt = "!"
            elseif FlapIdx > 20 and AirSpd > 85 then
            FlapResTxt = "!"
            elseif FlapIdx == 0 and AirSpd <= 100 then
            FlapResTxt = "."
            elseif FlapIdx > 0 and FlapIdx <= 20 and AirSpd <= 85 then
            FlapResTxt = "."

            else FlapResTxt = " "
            end
         end
    else FlapResTxt = " "
    end


    -- Flaps moving?
    if FlapIdx ~= FlpLvr then FlapMvTxt = "~"  FlapMvTxtshrt = "~"
    else FlapMvTxt = " "  FlapMvTxtshrt = "F"
    end


    -- ice building

    StrctIce = round2(ipc.readLvar("Structural_ice_buildup"))
    if StrctIce > 5 then
    IceTxtshrt = "i"..StrctIce .. "%"
    IceTxt = "Ice: " .. StrctIce .. "%"

    else
    IceTxtshrt = ""
    IceTxt = ""
    end

    ------------------------

    if _MCP1 () then
    FLIGHT_INFO1 = FlapMvTxtshrt .. "" .. FlpLvr .. FlapResTxt
    FLIGHT_INFO2 = IceTxtshrt

    elseif _MCP2 () then
    FLIGHT_INFO1 = "Flp:" .. FlapMvTxt .. "" .. FlpLvr .. FlapResTxt
    FLIGHT_INFO2 = IceTxt
    end


end

-- ## Autopilot ###############

function TW_ALT_plus ()

    TWalt = ipc.readLvar("Alt_alert_setting")
    ipc.sleep(10)
    ipc.writeLvar("Alt_alert_setting", TWalt+100)
    DspALT(TWalt/100)

end

function TW_ALT_plusfast ()

    TWalt = ipc.readLvar("Alt_alert_setting")

    ipc.sleep(10)
    ipc.writeLvar("Alt_alert_setting", TWalt+500)
    DspALT(TWalt/100)

end

function TW_ALT_minus ()

    TWalt = ipc.readLvar("Alt_alert_setting")
    ipc.sleep(10)
    ipc.writeLvar("Alt_alert_setting", TWalt-100)
    DspALT(TWalt/100)

end

function TW_ALT_minusfast ()

    TWalt = ipc.readLvar("Alt_alert_setting")
    ipc.sleep(10)
    ipc.writeLvar("Alt_alert_setting", TWalt-500)
    DspALT(TWalt/100)
end


function TWAutopilot_MASTER_toggle ()
    Twotter_Switchsound_1 ()
	_AP_MASTER ()
end

function TWAutopilot_MASTER_on ()
    Twotter_Switchsound_1 ()
	_AUTOPILOT_ON ()
end

function TWAutopilot_MASTER_off ()
    Twotter_Switchsound_1 ()
	_AUTOPILOT_OFF ()
end

function TW_AP_HDG_hold ()

	_AP_HDG_HOLD ()
    ipc.writeLvar("L:AP_HDG_Anim", 1)
    _sleep(50,100)
    ipc.writeLvar("L:AP_HDG_Anim", 0)
    --Twotter_Switchsound_2 ()
end

function TW_AP_ALT_hold ()

	_AP_ALT_HOLD ()
    ipc.writeLvar("L:AP_ALT_Anim", 1)
    ipc.writeLvar("L:AP_IAS_HOLD", 0)
    _sleep(50,100)
    ipc.writeLvar("L:AP_ALT_Anim", 0)
    --Twotter_Switchsound_2 ()
end

function TW_AP_NAV_hold ()

	_AP_NAV1_HOLD ()
    ipc.writeLvar("L:AP_NAV_Anim", 1)
    _sleep(50,100)
    ipc.writeLvar("L:AP_NAV_Anim", 0)
    --Twotter_Switchsound_2 ()
end

function TW_AP_IAS_hold ()
	if _tl("AP_IAS_HOLD", 0) then
       ipc.writeLvar("L:AP_IAS_HOLD", 1)
	else
       ipc.writeLvar("L:AP_IAS_HOLD", 0)
	end

    ipc.writeLvar("L:AP_IAS_Anim", 1)
    _sleep(50,100)
    ipc.writeLvar("L:AP_IAS_Anim", 0)
    --Twotter_Switchsound_2 ()
end

function TW_AP_APPR_hold ()

	_AP_APR_HOLD ()
    ipc.writeLvar("L:AP_APP_Anim", 1)
    _sleep(50,100)
    ipc.writeLvar("L:AP_APP_Anim", 0)
    --Twotter_Switchsound_2 ()
end

function TW_AP_BC_hold ()

	_AP_BC_HOLD ()
    ipc.writeLvar("L:AP_BC_Anim", 1)
    _sleep(50,100)
    ipc.writeLvar("L:AP_BC_Anim", 0)
    --Twotter_Switchsound_2 ()
end

function TW_AP_Yaw_on ()
    ipc.control(66069)
    DspShow ("Yaw", "on")
end

function TW_AP_Yaw_off ()
    ipc.control(66070)
    DspShow ("Yaw", "off")
end

function TW_AP_Yaw_toggle ()
    ipc.control(65793)
    DspShow ("Yaw", "dump")
end


function TW_AP_Pitch_thumbknob_up ()

    TWPitchVar = ipc.readLvar("AP_FD_Pitch_Mode_val")

    if TWPitchVar < 10 then
    TWPitchVar = TWPitchVar - 0.25
    elseif TWPitchVar >= 10 then
    TWPitchVar = 10
    end

    ipc.writeLvar("L:AP_FD_Pitch_Mode_val", TWPitchVar)

    ipc.writeLvar("L:AP_Pitch_thumbknob", -1)
    DspShow ("Ptch", (TWPitchVar)*-1)
end

function TW_AP_Pitch_thumbknob_dn ()

    TWPitchVar = ipc.readLvar("AP_FD_Pitch_Mode_val")

    if TWPitchVar > -10 then
    TWPitchVar = TWPitchVar + 0.25
    elseif TWPitchVar <= -10 then
    TWPitchVar = -10
    end

    ipc.writeLvar("L:AP_FD_Pitch_Mode_val", TWPitchVar)

    ipc.writeLvar("L:AP_Pitch_thumbknob", -1)
    DspShow ("Ptch", (TWPitchVar)*-1)
end

---
function TW_AP_Bank_thumbknob_show ()
     TWBankVar = ipc.readLvar("AP_FD_Lateral_Mode_val")

     if TWBankVar < 0 then
     TWBankTxt = round(TWBankVar)*-1
     TWBankSymb = "> "
     elseif TWBankVar >= 0 then
     TWBankTxt = round(TWBankVar)
     TWBankSymb = "< "
     end

     DspShow ("Bank", TWBankSymb .. TWBankTxt)
end


function TW_AP_Bank_thumbknob_right ()

    TWBankVar = ipc.readLvar("AP_FD_Lateral_Mode_val")

    if TWBankVar > -25 then
    TWBankVar = TWBankVar - 0.5
--    elseif TWBankVar >= -25 then
    elseif TWBankVar <= -25 then	-- MH fixed
    TWBankVar = -25
    end

    ipc.writeLvar("L:AP_FD_Lateral_Mode_val", TWBankVar)

    ipc.writeLvar("L:AP_Bank_thumbknob", -1)
    TW_AP_Bank_thumbknob_show ()
end

function TW_AP_Bank_thumbknob_left ()

    TWBankVar = ipc.readLvar("AP_FD_Lateral_Mode_val")

    if TWBankVar < 25 then
    TWBankVar = TWBankVar + 0.5
--    elseif TWBankVar <= 25 then
    elseif TWBankVar <= 25 then	-- MH fixed
    TWBankVar = 25
    end

    ipc.writeLvar("L:AP_FD_Lateral_Mode_val", TWBankVar)

    ipc.writeLvar("L:AP_Bank_thumbknob", -1)
    TW_AP_Bank_thumbknob_show ()
end



-- ## Electric ###############

function TW_DC_Master_sw_on ()
    ipc.writeLvar("L:TW_DC_Master_sw", 1)
    Twotter_Switchsound_1 ()
	DspShow ("DC ", "Mstr")
end

function TW_DC_Master_sw_off ()
    ipc.writeLvar("L:TW_DC_Master_sw", 0)
    Twotter_Switchsound_1 ()
	DspShow ("DC ", "Off  ")
end

function TW_DC_Master_sw_toggle ()
	if _tl("TW_DC_Master_sw", 0) then
       TW_DC_Master_sw_on ()
	else
       TW_DC_Master_sw_off ()
	end
end



function TW_DC_Source_sw_Batt ()
    DC_Source_var = "L:TW_DC_Source_sw"

    if ipc.readLvar(DC_Source_var) == 0 then
    ipc.writeLvar(DC_Source_var, 1)
    Twotter_Switchsound_1 ()
	DspShow ("DC ", "Batt  ")
    end
end

function TW_DC_Source_sw_Off ()
    DC_Source_var = "L:TW_DC_Source_sw"

    if ipc.readLvar(DC_Source_var) ~= 0 then
    ipc.writeLvar(DC_Source_var, 0)
    Twotter_Switchsound_1 ()
	DspShow ("DC ", "Off  ")
    end
end

function TW_DC_Source_sw_Ext ()
    DC_Source_var = "L:TW_DC_Source_sw"

    if ipc.readLvar(DC_Source_var) == 0 then
    ipc.writeLvar(DC_Source_var, -1)
    Twotter_Switchsound_1 ()
	DspShow ("DC ", "Ext  ")
    end
end

function TW_DC_Source_sw_Batt_toggle ()
    DC_Source_var = "L:TW_DC_Source_sw"

    if _tl(DC_Source_var, 1) then
       TW_DC_Source_sw_Off ()
    elseif _tl(DC_Source_var, 0) then
       TW_DC_Source_sw_Batt ()
	end
end

function TW_DC_Source_sw_Ext_toggle ()
    DC_Source_var = "L:TW_DC_Source_sw"

    if _tl(DC_Source_var, -1) then
       TW_DC_Source_sw_Off ()
	elseif _tl(DC_Source_var, 0) then
       TW_DC_Source_sw_Ext ()
	end
end

---
function TW_DC_Bustie_sw_norm ()
    ipc.writeLvar("L:TW_DC_Bustie_sw", 1)
    Twotter_Switchsound_1 ()
	DspShow ("Tie", "norm")
end

function TW_DC_Bustie_sw_open ()
    ipc.writeLvar("L:TW_DC_Bustie_sw", 0)
    Twotter_Switchsound_1 ()
	DspShow ("Tie", "open")
end


function TW_DC_Gen1_sw_on ()
    ipc.writeLvar("L:TW_DC_Gen1_sw", 1)
    Twotter_Switchsound_1 ()
	DspShow ("Gen1", "on")
end

function TW_DC_Gen1_sw_off ()
    ipc.writeLvar("L:TW_DC_Gen1_sw", 0)
    Twotter_Switchsound_1 ()
	DspShow ("Gen1", "off ")
end

function TW_DC_Gen1_reset ()
    if ipc.readLvar("L:TW_DC_Gen1_sw") == 1 then

    ipc.writeLvar("L:TW_DC_Gen1_sw", 2)
    Twotter_Switchsound_1 ()
    _sleep(100)
    ipc.writeLvar("L:TW_DC_Gen1_sw", 1)
    Twotter_Switchsound_1 ()
	DspShow ("Gen1", "rst")
    end
end

function TW_DC_Gen1_sw_toggle ()
	if _tl("TW_DC_Gen1_sw", 0) then
       TW_DC_Gen1_sw_on ()
	else
       TW_DC_Gen1_sw_off ()
	end
end

--

function TW_DC_Gen2_sw_on ()
    ipc.writeLvar("L:TW_DC_Gen2_sw", 1)
    Twotter_Switchsound_1 ()
	DspShow ("Gen2", "on")
end

function TW_DC_Gen2_sw_off ()
    ipc.writeLvar("L:TW_DC_Gen2_sw", 0)
    Twotter_Switchsound_1 ()
	DspShow ("Gen2", "off ")
end

function TW_DC_Gen2_reset ()
    if ipc.readLvar("L:TW_DC_Gen2_sw") == 1 then

    ipc.writeLvar("L:TW_DC_Gen2_sw", 2)
    Twotter_Switchsound_1 ()
    _sleep(100)
    ipc.writeLvar("L:TW_DC_Gen2_sw", 1)
    Twotter_Switchsound_1 ()
	DspShow ("Gen2", "rst")
    end
end

function TW_DC_Gen2_sw_toggle ()
	if _tl("TW_DC_Gen2_sw", 0) then
       TW_DC_Gen2_sw_on ()
	else
       TW_DC_Gen2_sw_off ()
	end
end


---

function TW_DC_Both_Gen_on ()
    TW_DC_Gen1_sw_on ()
    _sleep(100,200)
    TW_DC_Gen2_sw_on ()
end

function TW_DC_Both_Gen_off ()
    TW_DC_Gen1_sw_off ()
    _sleep(100,200)
    TW_DC_Gen2_sw_off ()
end


function TW_DC_Both_Gen_toggle ()
	if _tl("TW_DC_Gen1_sw", 0) then
       TW_DC_Both_Gen_on ()
	else
       TW_DC_Both_Gen_off ()
	end
end

function TW_AC_Inv_sw_1 ()
    ipc.writeLvar("L:TW_AC_Inv_sw", 0)
    Twotter_Switchsound_1 ()
	DspShow ("Inv", "1")
end

function TW_AC_Inv_sw_2 ()
    ipc.writeLvar("L:TW_AC_Inv_sw", 1)
    Twotter_Switchsound_1 ()
	DspShow ("Inv", "2")
end


function TW_AC_Inv_sw_toggle ()
	if _tl("TW_AC_Inv_sw", 0) then
      TW_AC_Inv_sw_2 ()
	else
      TW_AC_Inv_sw_1 ()
	end
end

-- ## Engine start ###############

function Starter_Sw_left ()
    ipc.writeLvar("L:Starter_Sw", -1)
    Twotter_Switchsound_1 ()
	DspShow ("Strt", "left")
end

function Starter_Sw_off ()
    ipc.writeLvar("L:Starter_Sw", 0)
    Twotter_Switchsound_1 ()
	DspShow ("Strt", "off ")
end

function Starter_Sw_right ()
    ipc.writeLvar("L:Starter_Sw", 1)
    Twotter_Switchsound_1 ()
	DspShow ("Strt", "rght")
end

function Starter_Sw_toggle_left ()
	if _tl("Starter_Sw", 0) then
       Starter_Sw_left ()
	elseif _tl("Starter_Sw", -1) then
       Starter_Sw_off ()
	end
end

function Starter_Sw_toggle_right ()
	if _tl("Starter_Sw", 0) then
       Starter_Sw_right ()
	elseif _tl("Starter_Sw", 1) then
       Starter_Sw_off ()
	end
end


function Ign_Mode_Sw_manual ()

    ipc.writeLvar("L:Twot_GuardSound", 1)
    ipc.writeLvar("L:Ign_Mode_Guard", 1)
    _sleep(250)
    ipc.writeLvar("L:Ign_Mode_Sw", 1)
    Twotter_Switchsound_1 ()
	DspShow ("IgnM", "Man")
end

function Ign_Mode_Sw_normal ()

    ipc.writeLvar("L:Ign_Mode_Sw", 0)
    Twotter_Switchsound_1 ()
	DspShow ("IgnM", "Norm")
    _sleep(250)
    ipc.writeLvar("L:Twot_GuardSound", 1)
    ipc.writeLvar("L:Ign_Mode_Guard", 0)
end

function Ign_Mode_Sw_toggle ()
	if _tl("Ign_Mode_Sw", 0) then
       Ign_Mode_Sw_manual ()
	else
       Ign_Mode_Sw_normal ()
	end
end

---

function Ign_Eng1_Sw_2 ()
    ipc.writeLvar("L:Ign_Eng1_Sw", 1)
    Twotter_Switchsound_1 ()
	DspShow ("Ign1", "2")
end

function Ign_Eng1_Sw_both ()
    ipc.writeLvar("L:Ign_Eng1_Sw", 0)
    Twotter_Switchsound_1 ()
	DspShow ("Ign1", "both")
end

function Ign_Eng1_Sw_1 ()
    ipc.writeLvar("L:Ign_Eng1_Sw", -1)
    Twotter_Switchsound_1 ()
	DspShow ("Ign1", "1")
end


function Ign_Eng2_Sw_2 ()
    ipc.writeLvar("L:Ign_Eng2_Sw", 1)
    Twotter_Switchsound_1 ()
	DspShow ("Ign2", "2")
end

function Ign_Eng2_Sw_both ()
    ipc.writeLvar("L:Ign_Eng2_Sw", 0)
    Twotter_Switchsound_1 ()
	DspShow ("Ign2", "both")
end

function Ign_Eng2_Sw_1 ()
    ipc.writeLvar("L:Ign_Eng2_Sw", -1)
    Twotter_Switchsound_1 ()
	DspShow ("Ign2", "1")
end


-- ## Radio etc ###############

function GPS_Power_on ()
    ipc.writeLvar("L:GPS_Power", 1)
    Twotter_Switchsound_1 ()
	DspShow ("GPS ", "on")
end

function GPS_Power_off ()
    ipc.writeLvar("L:GPS_Power", 0)
    Twotter_Switchsound_1 ()
	DspShow ("GPS ", "off  ")
end

function GPS_Power_toggle ()
	if _tl("GPS_Power", 0) then
       GPS_Power_on ()
	else
       GPS_Power_off ()
	end
end

--

function ComNav2_Power_on ()
    ipc.writeLvar("L:ComNav2_Power", 1)
    Twotter_Switchsound_1 ()
	DspShow ("Com2", "on")
end

function ComNav2_Power_off ()
    ipc.writeLvar("L:ComNav2_Power", 0)
    Twotter_Switchsound_1 ()
	DspShow ("Com2", "off  ")
end

function ComNav2_Power_toggle ()
	if _tl("ComNav2_Power", 0) then
       ComNav2_Power_on ()
	else
       ComNav2_Power_off ()
	end
end



function dme_power_on ()
    ipc.writeLvar("L:dme_power", 1)
    --Twotter_Switchsound_1 ()
	DspShow ("DME", "on")
end

function dme_power_off ()
    ipc.writeLvar("L:dme_power", 0)
    --Twotter_Switchsound_1 ()
	DspShow ("DME", "off  ")
end

function dme_power_toggle ()
	if _tl("dme_power", 0) then
       dme_power_on ()
	else
       dme_power_off ()
	end
end


function Mod_ADF_Power_on ()
    ipc.writeLvar("L:Mod_ADF_Power", 1)
    Twotter_Switchsound_1 ()
	DspShow ("ADF", "on")
end

function Mod_ADF_Power_off ()
    ipc.writeLvar("L:Mod_ADF_Power", 0)
    Twotter_Switchsound_1 ()
	DspShow ("ADF", "off  ")
end

function Mod_ADF_Power_toggle ()
	if _tl("Mod_ADF_Power", 0) then
       Mod_ADF_Power_on ()
	else
       Mod_ADF_Power_off ()
	end
end

---

function Twot_XPDR_show ()

    XPDRvar = ipc.readLvar("L:Xponder_mode")

    if XPDRvar == 0 then
    XPDRtext = "off"
    elseif XPDRvar == 1 then
    XPDRtext = "sby"
    elseif XPDRvar == 2 then
    XPDRtext = "test"
    elseif XPDRvar == 3 then
    XPDRtext = "gnd"
    elseif XPDRvar == 4 then
    XPDRtext = "on"
    elseif XPDRvar == 5 then
    XPDRtext = "alt"
    end
    if _MCP1() then
      DspShow("XPDR", XPDRtext)
    elseif _MCP2() then

    DspRadioShort(XPDRtext)
    end
end

function Twot_XPDR_off ()

      ipc.writeLvar("L:Xponder_mode", 0)
      Twot_XPDR_show ()
end

function Twot_XPDR_sby ()

      ipc.writeLvar("L:Xponder_mode", 1)
      Twot_XPDR_show ()
end

function Twot_XPDR_test ()

      ipc.writeLvar("L:Xponder_mode", 2)
      Twot_XPDR_show ()
end

function Twot_XPDR_gnd ()

      ipc.writeLvar("L:Xponder_mode", 3)
      Twot_XPDR_show ()
end

function Twot_XPDR_on ()

      ipc.writeLvar("L:Xponder_mode", 4)
      Twot_XPDR_show ()
end

function Twot_XPDR_alt ()

      ipc.writeLvar("L:Xponder_mode", 5)
      Twot_XPDR_show ()
end


-- XPDR inc

function Twot_XPDR_inc ()

	i = ipc.readLvar("L:Xponder_mode")

	if i < 5 then
	ipc.writeLvar("L:Xponder_mode", i+1)

	end

	if i == 5 then
	ipc.writeLvar("L:Xponder_mode", 5)

	end
    Twot_XPDR_show ()
end

-- XPDR dec

function Twot_XPDR_dec ()

	i = ipc.readLvar("L:Xponder_mode")

	if i > 0 then
	ipc.writeLvar("L:Xponder_mode", i-1)

	end

	if i == 0 then
	ipc.writeLvar("L:Xponder_mode", 0)

	end
    Twot_XPDR_show ()
end


function Twot_All_Avionics_on ()
    TW_DC_Master_sw_on ()
    _sleep(150, 400)
    GPS_Power_on ()
    _sleep(100, 200)
    ComNav2_Power_on ()
    _sleep(100, 200)
    dme_power_on ()
    _sleep(100, 200)
    Mod_ADF_Power_on ()
    _sleep(100, 200)
    Twot_XPDR_on ()
end

function Twot_All_Avionics_off ()

    Twot_XPDR_off ()
    _sleep(100, 200)
    Mod_ADF_Power_off ()
    _sleep(100, 200)
    dme_power_off ()
    _sleep(100, 200)
    ComNav2_Power_off ()
    _sleep(100, 200)
    GPS_Power_off ()
    _sleep(200, 500)
    TW_DC_Master_sw_off ()
end

function Twot_All_Avionics_toggle ()

	if _tl("TW_DC_Master_sw", 0) then
          TW_DC_Master_sw_toggle ()
    _sleep(150, 400)
    GPS_Power_toggle ()
    _sleep(100, 200)
    ComNav2_Power_toggle ()
    _sleep(100, 200)
    dme_power_toggle ()
    _sleep(100, 200)
    Mod_ADF_Power_toggle ()
    _sleep(100, 200)
	if _tl("Xponder_mode", 0) then
      Twot_XPDR_on ()
	else
       Twot_XPDR_off ()
	end
	else

	if _tl("Xponder_mode", 0) then
      Twot_XPDR_on ()
	else
       Twot_XPDR_off ()
	end
    _sleep(100, 200)
    Mod_ADF_Power_off ()
    _sleep(100, 200)
    dme_power_off ()
    _sleep(100, 200)
    ComNav2_Power_off ()
    _sleep(100, 200)
    GPS_Power_off ()
    _sleep(200, 500)
    TW_DC_Master_sw_off ()
    end
end


-- ## External Lights ###############

function Lights_LANDING_on ()

	ipc.setbitsUW("0D0C", 4)
	DspShow ("LNDG", "On  ")
    Twotter_Switchsound_1 ()
end

function Lights_LANDING_off ()

	ipc.clearbitsUW("0D0C", 4)
	DspShow ("LNDG", "Off ")
    Twotter_Switchsound_1 ()
end

function Lights_LANDING_toggle ()

    if logic.And(ipc.readUW("0D0C"), 4) == 4 then
		Lights_LANDING_off ()
	else
		Lights_LANDING_on ()
	end
end


function Lights_TAXI_on ()

	ipc.setbitsUW("0D0C", 8)
	DspShow ("TAXI", "On  ")
    Twotter_Switchsound_1 ()
end

function Lights_TAXI_off ()

	ipc.clearbitsUW("0D0C", 8)
	DspShow ("TAXI", "Off ")
    Twotter_Switchsound_1 ()
end

function Lights_TAXI_toggle ()

    if logic.And(ipc.readUW("0D0C"), 8) == 8 then
		Lights_TAXI_off ()
	else
		Lights_TAXI_on ()
	end
end



function Lights_NAV_on ()

	ipc.setbitsUW("0D0C", 1)
	DspShow ("NAV ", "On  ")
    Twotter_Switchsound_1 ()
end

function Lights_NAV_off ()

	ipc.clearbitsUW("0D0C", 1)
	DspShow ("NAV ", "Off ")
    Twotter_Switchsound_1 ()
end

function Lights_NAV_toggle ()

    if logic.And(ipc.readUW("0D0C"), 1) == 1 then
		Lights_NAV_off ()
	else
		Lights_NAV_on ()
	end
end



function Lights_BEACON_on ()

	ipc.setbitsUW("0D0C", 2)
	DspShow ("BCN ", "On  ")
    Twotter_Switchsound_1 ()
end

function Lights_BEACON_off ()

	ipc.clearbitsUW("0D0C", 2)
	DspShow ("BCN ", "Off ")
    Twotter_Switchsound_1 ()
end

function Lights_BEACON_toggle ()

    if logic.And(ipc.readUW("0D0C"), 2) == 2 then
		Lights_BEACON_off ()
	else
		Lights_BEACON_on ()
	end
end

function Lights_STROBE_on ()

	ipc.setbitsUW("0D0C", 16)
	DspShow ("STRB", "On ")
    Twotter_Switchsound_1 ()
end

function Lights_STROBE_off ()

	ipc.clearbitsUW("0D0C", 16)
	DspShow ("STRB", "Off ")
    Twotter_Switchsound_1 ()
end

function Lights_STROBE_toggle ()

    if logic.And(ipc.readUW("0D0C"), 16) == 16 then
		Lights_STROBE_off ()
	else
		Lights_STROBE_on ()
	end
end

-- ## Internal lights ###############

function Lights_CABIN_on ()

	ipc.setbitsUW("0D0C", 512)
	DspShow ("FLD ", "On  ")
    Twotter_Switchsound_1 ()
end

function Lights_CABIN_off ()

	ipc.clearbitsUW("0D0C", 512)
	DspShow ("FLD ", "Off ")
    Twotter_Switchsound_1 ()
end

function Lights_CABIN_toggle ()

    if logic.And(ipc.readUW("0D0C"), 512) == 512 then
		Lights_CABIN_off ()
	else
		Lights_CABIN_on ()
	end
end

function Lights_PANEL_on ()

	ipc.setbitsUW("0D0C", 32)
	DspShow ("PANL", "On  ")
     Twotter_Switchsound_1 ()
end

function Lights_PANEL_off ()

	ipc.clearbitsUW("0D0C", 32)
	DspShow ("PANL", "Off ")
    Twotter_Switchsound_1 ()
end

function Lights_PANEL_toggle ()

    if logic.And(ipc.readUW("0D0C"), 32) == 32 then
		Lights_PANEL_off ()
	else
		Lights_PANEL_on ()
	end
end

function Light_Emer_disarm ()
    ipc.writeLvar("L:Light_Emer", -1)
    DspShow ("Emer", "DArm")
    Twotter_Switchsound_1 ()
end

function Light_Emer_test ()
    ipc.writeLvar("L:Light_Emer", 0)
    DspShow ("Emer", "test")
    Twotter_Switchsound_1 ()
end

function Light_Emer_arm ()
    ipc.writeLvar("L:Light_Emer", 1)
    DspShow ("Emer", "arm")
    Twotter_Switchsound_1 ()
end

-----------

function No_Smoking_sw_on ()
    ipc.writeLvar("L:No_Smoking_sw", 1)
    DspShow ("Smk", "on")
    Twotter_Switchsound_1 ()
end

function No_Smoking_sw_off ()
    ipc.writeLvar("L:No_Smoking_sw", 0)
    DspShow ("Smk", "off")
    Twotter_Switchsound_1 ()
end

function No_Smoking_sw_toggle ()
	if _tl("No_Smoking_sw", 0) then
       No_Smoking_sw_on ()
	else
       No_Smoking_sw_off ()
	end
end

---

-----------

function Seatbelts_sw_on ()
    ipc.writeLvar("L:Seatbelts_sw", 1)
    DspShow ("Seat", "on")
    Twotter_Switchsound_1 ()
end

function Seatbelts_sw_off ()
    ipc.writeLvar("L:Seatbelts_sw", 0)
    DspShow ("Seat", "off")
    Twotter_Switchsound_1 ()
end

function Seatbelts_sw_toggle ()
	if _tl("Seatbelts_sw", 0) then
       Seatbelts_sw_on ()
	else
       Seatbelts_sw_off ()
	end
end

---

function Compartment_Lights_toggle ()
    ipc.control(66377)
    DspShow ("Comp", "")
    Twotter_Switchsound_1 ()
end


-- ## Fuel ###############

function Fuel_ind_but_press ()
    ipc.writeLvar("L:Fuel_ind_but", 1)
    Twotter_Switchsound_1 ()
end

function Fuel_ind_but_release ()
    ipc.writeLvar("L:Fuel_ind_but", 0)
    Twotter_Switchsound_1 ()
end



function FuelBoost_AFT_on ()
    ipc.writeLvar("L:FuelBoost_AFT", 1)
    Twotter_Switchsound_1 ()
	DspShow ("AFT", "on")
end

function FuelBoost_AFT_off ()
    ipc.writeLvar("L:FuelBoost_AFT", 0)
    Twotter_Switchsound_1 ()
	DspShow ("AFT", "off ")
end

function FuelBoost_AFT_test ()
    ipc.writeLvar("L:FuelBoost_AFT", -1)
    Twotter_Switchsound_1 ()
	DspShow ("AFT", "test")
end

function FuelBoost_AFT_toggle ()
	if _tl("FuelBoost_AFT", 0) then
       FuelBoost_AFT_on ()
	else
       FuelBoost_AFT_off ()
	end
end

---

function FuelBoost_FWD_on ()
    ipc.writeLvar("L:FuelBoost_FWD", 1)
    Twotter_Switchsound_1 ()
	DspShow ("FWD", "on")
end

function FuelBoost_FWD_off ()
    ipc.writeLvar("L:FuelBoost_FWD", 0)
    Twotter_Switchsound_1 ()
	DspShow ("FWD", "off ")
end

function FuelBoost_FWD_test ()
    ipc.writeLvar("L:FuelBoost_FWD", -1)
    Twotter_Switchsound_1 ()
	DspShow ("FWD", "test")
end

function FuelBoost_FWD_toggle ()
	if _tl("FuelBoost_FWD", 0) then
       FuelBoost_FWD_on ()
	else
       FuelBoost_FWD_off ()
	end
end

---

function StdbyFuelBoost_AFT_on ()
    ipc.writeLvar("L:StdbyFuelBoost_AFT", 1)
    Twotter_Switchsound_1 ()
	DspShow ("AFT", "on")
end

function StdbyFuelBoost_AFT_off ()
    ipc.writeLvar("L:StdbyFuelBoost_AFT", 0)
    Twotter_Switchsound_1 ()
	DspShow ("AFT", "off ")
end

function StdbyFuelBoost_AFT_toggle ()
	if _tl("StdbyFuelBoost_AFT", 0) then
       StdbyFuelBoost_AFT_on ()
	else
       StdbyFuelBoost_AFT_off ()
	end
end

---

function StdbyFuelBoost_FWD_on ()
    ipc.writeLvar("L:StdbyFuelBoost_FWD", 1)
    Twotter_Switchsound_1 ()
	DspShow ("FWD", "on")
end

function StdbyFuelBoost_FWD_off ()
    ipc.writeLvar("L:StdbyFuelBoost_FWD", 0)
    Twotter_Switchsound_1 ()
	DspShow ("FWD", "off ")
end

function StdbyFuelBoost_FWD_toggle ()
	if _tl("StdbyFuelBoost_FWD", 0) then
       StdbyFuelBoost_FWD_on ()
	else
       StdbyFuelBoost_FWD_off ()
	end
end


-- ## Wiper ###############

function Twot_Wiper_on ()
    ipc.writeLvar("L:Windowwiper_Mode", 1)
    Twotter_Switchsound_1 ()
    DspShow ("Wipr", "on")
end

function Twot_Wiper_off ()
    ipc.writeLvar("L:Windowwiper_Mode", 0)
    Twotter_Switchsound_1 ()
    DspShow ("Wipr", "off")
end

function Twot_Wiper_park ()
    ipc.writeLvar("L:Windowwiper_Mode", -1)
    Twotter_Switchsound_1 ()
    DspShow ("Wipr", "park")
end

function Twot_Wiper_toggle ()
	if _tl("Windowwiper_Mode", 1) then
       Twot_Wiper_park ()
	else
       Twot_Wiper_on ()
	end

end

function Twot_Wiper_fast ()
    ipc.writeLvar("L:Twot_wiperspeed", 1)
    Twotter_Switchsound_1 ()
    DspShow ("Wipr", "fast")
end

function Twot_Wiper_slow ()
    ipc.writeLvar("L:Twot_wiperspeed", 0)
    Twotter_Switchsound_1 ()
    DspShow ("Wipr", "slow")
end

function Twot_Wiper_toggle ()
	if _tl("Twot_wiperspeed", 0) then
       Twot_Wiper_fast ()
	else
       Twot_Wiper_slow ()
	end

end

-- ## Deice ###############

function Twot_Pitot_on ()
    ipc.control(66074, 1)
    DspShow ("Ptot", "on")
    Twotter_Switchsound_1 ()
end

function Twot_Pitot_off ()
    ipc.control(66074, 0)
    DspShow ("Ptot", "off")
    Twotter_Switchsound_1 ()
end

--------

function Windowheat_Sw_on ()
    ipc.writeLvar("L:Windowheat_Sw", 1)
    DspShow ("Heat", "on")
    Twotter_Switchsound_1 ()
end

function Windowheat_Sw_off ()
    ipc.writeLvar("L:Windowheat_Sw", 0)
    DspShow ("Heat", "off")
    Twotter_Switchsound_1 ()
end

function Windowheat_Sw_toggle ()
	if _tl("Windowheat_Sw", 0) then
       Windowheat_Sw_on ()
	else
      Windowheat_Sw_off ()
	end
end

function PropDeice_on ()
    ipc.writeLvar("L:PropDeice", 1)
    DspShow ("Prop", "on")
    Twotter_Switchsound_1 ()
end

function PropDeice_off ()
    ipc.writeLvar("L:PropDeice", 0)
    DspShow ("Prop", "off")
    Twotter_Switchsound_1 ()
end

function PropDeice_toggle ()
	if _tl("PropDeice", 0) then
       PropDeice_on ()
	else
      PropDeice_off ()
	end
end

--------

function IntakeDeice_on ()
    ipc.writeLvar("L:IntakeDeice", 1)
    DspShow ("Intk", "on")
    Twotter_Switchsound_1 ()
end

function IntakeDeice_off ()
    ipc.writeLvar("L:IntakeDeice", 0)
    DspShow ("Intk", "off")
    Twotter_Switchsound_1 ()
end

function IntakeDeice_toggle ()
	if _tl("IntakeDeice", 0) then
       IntakeDeice_on ()
	else
      IntakeDeice_off ()
	end
end

--------

function Wing_Inspection_lights_toggle ()
    ipc.control(66378)
    DspShow ("Wlgt", "")
    Twotter_Switchsound_1 ()
end

--------

function Intake_Deflector_Sw_retract ()
    ipc.writeLvar("L:Intake_Deflector_Sw", -1)
    DspShow ("Intk", "retr")
    Twotter_Switchsound_1 ()
end

function Intake_Deflector_Sw_off ()
    ipc.writeLvar("L:Intake_Deflector_Sw", 0)
    DspShow ("Intk", "off")
    Twotter_Switchsound_1 ()
end

function Intake_Deflector_Sw_extend ()
    ipc.writeLvar("L:Intake_Deflector_Sw", 1)
    DspShow ("Intk", "extd")
    Twotter_Switchsound_1 ()
end

-- ## Deicer Boots ###############

function AI_ManAuto_sw_auto ()
    ipc.writeLvar("L:AI_ManAuto_sw", 1)
    DspShow ("AIce", "auto")
    Twotter_Switchsound_1 ()
end

function AI_ManAuto_sw_off ()
    ipc.writeLvar("L:AI_ManAuto_sw", 0)
    DspShow ("AIce", "off")
    Twotter_Switchsound_1 ()
end

function AI_ManAuto_sw_manual ()
    ipc.writeLvar("L:AI_ManAuto_sw", -1)
    DspShow ("AIce", "manu")
    Twotter_Switchsound_1 ()
end


function AI_SlowFast_sw_fast ()
    ipc.writeLvar("L:AI_SlowFast_sw", 1)
    DspShow ("AIce", "fast")
    Twotter_Switchsound_1 ()
end

function AI_SlowFast_sw_slow ()
    ipc.writeLvar("L:AI_SlowFast_sw", 0)
    DspShow ("AIce", "slow")
    Twotter_Switchsound_1 ()
end

function AI_WingInOut_sw_outer ()
    ipc.writeLvar("L:AI_WingInOut_sw", 1)
    DspShow ("WIce", "out")
    Twotter_Switchsound_1 ()
end

function AI_WingInOut_sw_release ()
    ipc.writeLvar("L:AI_WingInOut_sw", 0)
    DspShow ("WIce", "")
    Twotter_Switchsound_1 ()
end

function AI_WingInOut_sw_inner ()
    ipc.writeLvar("L:AI_WingInOut_sw", -1)
    DspShow ("WIce", "inn")
    Twotter_Switchsound_1 ()
end

function AI_Stab_LR_sw_right ()
    ipc.writeLvar("L:AI_Stab_LR_sw", 1)
    DspShow ("Stab", "rght")
    Twotter_Switchsound_1 ()
end

function AI_Stab_LR_sw_release ()
    ipc.writeLvar("L:AI_Stab_LR_sw", 0)
    DspShow ("Stab", "")
    Twotter_Switchsound_1 ()
end

function AI_Stab_LR_sw_left ()
    ipc.writeLvar("L:AI_Stab_LR_sw", -1)
    DspShow ("Stab", "left")
    Twotter_Switchsound_1 ()
end


-- ## Fire ###############

function FuelEmerg_L_on ()
    ipc.writeLvar("L:FuelEmerg_L", 1)
    DspShow ("Fuel", "L on")
    Twotter_Switchsound_1 ()
end

function FuelEmerg_L_off ()
    ipc.writeLvar("L:FuelEmerg_L", 0)
    DspShow ("Fuel", "Loff")
    Twotter_Switchsound_1 ()
end

function FuelEmerg_L_toggle ()
	if _tl("FuelEmerg_L", 0) then
       FuelEmerg_L_on ()
	else
       FuelEmerg_L_off ()
	end
end


function FuelEmerg_R_on ()
    ipc.writeLvar("L:FuelEmerg_R", 1)
    DspShow ("Fuel", "R on")
    Twotter_Switchsound_1 ()
end

function FuelEmerg_R_off ()
    ipc.writeLvar("L:FuelEmerg_R", 0)
    DspShow ("Fuel", "Roff")
    Twotter_Switchsound_1 ()
end

function FuelEmerg_R_toggle ()
	if _tl("FuelEmerg_R", 0) then
       FuelEmerg_R_on ()
	else
       FuelEmerg_R_off ()
	end
end

function FireTest_sw_on ()
    ipc.writeLvar("L:FireTest_sw", 1)
    DspShow ("Fire", "on")
    Twotter_Switchsound_1 ()
end

function FireTest_sw_off ()
    ipc.writeLvar("L:FireTest_sw", 0)
    DspShow ("Fire", "off")
    Twotter_Switchsound_1 ()
end

function FireTest_sw_toggle ()
	if _tl("FireTest_sw", 0) then
       FireTest_sw_on ()
	else
       FireTest_sw_off ()
	end
end




-- ## other ###############

function BleedairL_on ()
    ipc.writeLvar("L:BleedairL", 1)
    DspShow ("Bled", "L on")
    Twotter_Switchsound_1 ()
end

function BleedairL_off ()
    ipc.writeLvar("L:BleedairL", 0)
    DspShow ("Bled", "Loff")
    Twotter_Switchsound_1 ()
end

function BleedairL_toggle ()
	if _tl("BleedairL", 0) then
       BleedairL_on ()
	else
       BleedairL_off ()
	end
end


function BleedairR_on ()
    ipc.writeLvar("L:BleedairR", 1)
    DspShow ("Bled", "R on")
    Twotter_Switchsound_1 ()
end

function BleedairR_off ()
    ipc.writeLvar("L:BleedairR", 0)
    DspShow ("Bled", "Roff")
    Twotter_Switchsound_1 ()
end

function BleedairR_toggle ()
	if _tl("BleedairR", 0) then
       BleedairR_on ()
	else
       BleedairR_off ()
	end
end

function Twot_Gustlock_on ()
    ipc.writeLvar("L:Twot_Gustlock", 1)
    --Twotter_Switchsound_1 ()
	DspShow ("Gust", "on")
end

function Twot_Gustlock_off ()
    ipc.writeLvar("L:Twot_Gustlock", 0)
    --Twotter_Switchsound_1 ()
	DspShow ("Gust", "off  ")
end

function Twot_Gustlock_toggle ()
	if _tl("Twot_Gustlock", 0) then
       Twot_Gustlock_on ()
	else
       Twot_Gustlock_off ()
	end
end

-- Everything below here added by Mark Hurst, July 2014

-- MH local functions

local function Round(nVal)
	return math.floor(nVal + 0.5)
end

local function nIncrementMod(nCurrentVal, nStep, nLimit)
	-- Modulo increment (wraps around from zero)
	-- NOTE: adding the extra increment allows us to use a negative step for decrementing.
	return (nCurrentVal + nStep + nLimit) % nLimit
end

local function nToggleVal(nCurrVal, nVal1, nVal2)
	if nCurrVal == nVal1 then
		return nVal2
	else
		return nVal1
	end
end

-- ## MH GNS530 ###############

function GNS530_Power_ON()
    ipc.writeLvar("L:GPS_Power", 1)
end

function GNS530_Power_OFF()
    ipc.writeLvar("L:GPS_Power", 0)
end

function GNS530_Toggle_2D_GPS()
	_PANEL_ID_TOGGLE(34567)
end

function GNS530_Close_2D_GPS()
	_PANEL_ID_CLOSE(34567)
end

function GNS530_GPS_Outer_Knob_Left()
	_GPS_GROUP_KNOB_DEC()
end

function GNS530_GPS_Outer_Knob_Right()
	_GPS_GROUP_KNOB_INC()
end

function GNS530_GPS_Inner_Knob_Left()
	_GPS_PAGE_KNOB_DEC()
end

function GNS530_GPS_Inner_Knob_Right()
	_GPS_PAGE_KNOB_INC()
end

function GNS530_GPS_Push_Cursor()
	_GPS_CURSOR_BUTTON()
end

function GNS530_Increase_Range()
	_GPS_ZOOMOUT_BUTTON()
end

function GNS530_Decrease_Range()
	_GPS_ZOOMIN_BUTTON()
end

function GNS530_Direct_To()
	_GPS_DIRECTTO_BUTTON()
end

function GNS530_MENU()
	_GPS_MENU_BUTTON()
end

function GNS530_CLR()
	_GPS_CLEAR_BUTTON()
end

function GNS530_ENT()
	_GPS_ENTER_BUTTON()
end

function GNS530_CDI()
	_TOGGLE_GPS_DRIVES_NAV1()
end

function GNS530_OBS()
	_GPS_OBS_BUTTON()
end

function GNS530_MSG()
	_GPS_MSG_BUTTON()
end

function GNS530_FPL()
	_GPS_FLIGHTPLAN_BUTTON()
end

function GNS530_VNAV()
	_GPS_TERRAIN_BUTTON()
end

function GNS530_PROC()
	_GPS_PROCEDURE_BUTTON()
end

function GNS530_Radio_Outer_Knob_Left()
	if ipc.ReadLvar("L:GPS_ComNav_select") == 0 then
		_COM_RADIO_WHOLE_DEC()
	else
		_NAV1_RADIO_WHOLE_DEC()
	end
end

function GNS530_Radio_Outer_Knob_Right()
	if ipc.ReadLvar("L:GPS_ComNav_select") == 0 then
		_COM_RADIO_WHOLE_INC()
	else
		_NAV1_RADIO_WHOLE_INC()
	end
end

function GNS530_Radio_Inner_Knob_Left()
	if ipc.ReadLvar("L:GPS_ComNav_select") == 0 then
		_COM_RADIO_FRACT_DEC()
	else
		_NAV1_RADIO_FRACT_DEC()
	end
end

function GNS530_Radio_Inner_Knob_Right()
	if ipc.ReadLvar("L:GPS_ComNav_select") == 0 then
		_COM_RADIO_FRACT_INC()
	else
		_NAV1_RADIO_FRACT_INC()
	end
end

function GNS530_COM_NAV_Swap()
    ipc.writeLvar("L:GPS_ComNav_select", nToggleVal(ipc.readLvar("L:GPS_ComNav_select"), 0, 1))
end

function GNS530_NAV1_Ident_ON()
	ipc.WriteLvar("L:GPS_Com1_Ident", 1)
end

function GNS530_NAV1_Ident_OFF()
	ipc.WriteLvar("L:GPS_Com1_Ident", 0)
end

function GNS530_COM1_Swap_Freq()
	_COM_STBY_RADIO_SWAP()
end

function GNS530_NAV1_Swap_Freq()
	_NAV1_RADIO_SWAP()
end

-- ## MH Speaker Panel ###############

function SPK_COM1_ON()
    ipc.writeLvar("L:ICS_Com1", 1)
end

function SPK_COM_OFF()
    ipc.writeLvar("L:ICS_Com1", 0)
end

function SPK_COM1_Toggle()
    ipc.writeLvar("L:ICS_Com1", nToggleVal(ipc.readLvar("L:ICS_Com2"), 0, 1))
end

function SPK_COM2_ON()
    ipc.writeLvar("L:ICS_Com2", 1)
end

function SPK_COM2_OFF()
    ipc.writeLvar("L:ICS_Com2", 0)
end

function SPK_COM2_Toggle()
    ipc.writeLvar("L:ICS_Com2", nToggleVal(ipc.readLvar("L:ICS_Com2"), 0, 1))
end

function SPK_NAV1_ON()
    ipc.writeLvar("L:ICS_VOR1", 1)
end

function SPK_NAV1_OFF()
    ipc.writeLvar("L:ICS_VOR1", 0)
end

function SPK_NAV1_Toggle()
    ipc.writeLvar("L:ICS_VOR1", nToggleVal(ipc.readLvar("L:ICS_VOR1"), 0, 1))
end

function SPK_NAV2_ON()
    ipc.writeLvar("L:ICS_VOR2", 1)
end

function SPK_NAV2_OFF()
    ipc.writeLvar("L:ICS_VOR2", 0)
end

function SPK_NAV2_Toggle()
    ipc.writeLvar("L:ICS_VOR2", nToggleVal(ipc.readLvar("L:ICS_VOR2"), 0, 1))
end

function SPK_DME_ON()
    ipc.writeLvar("L:ICS_DME2", 1)
end

function SPK_DME_OFF()
    ipc.writeLvar("L:ICS_DME2", 0)
end

function SPK_DME_Toggle()
    ipc.writeLvar("L:ICS_DME2", nToggleVal(ipc.readLvar("L:ICS_DME2"), 0, 1))
end

function SPK_MKR_ON()
    ipc.writeLvar("L:ICS_MRK", 1)
end

function SPK_MKR_OFF()
    ipc.writeLvar("L:ICS_MRK", 0)
end

function SPK_MKR_Toggle()
    ipc.writeLvar("L:ICS_MRK", nToggleVal(ipc.readLvar("L:ICS_MRK"), 0, 1))
end

function SPK_ADF_ON()
    ipc.writeLvar("L:ICS_ADF", 1)
end

function SPK_ADF_OFF()
    ipc.writeLvar("L:ICS_ADF", 0)
end

function SPK_ADF_Toggle()
    ipc.writeLvar("L:ICS_ADF", nToggleVal(ipc.readLvar("L:ICS_ADF"), 0, 1))
end

function SPK_Transmit_COM1()
    ipc.writeLvar("L:ICS_com_Transmit", 0)
	SPK_COM1_ON()
end

function SPK_Transmit_COM2()
    ipc.writeLvar("L:ICS_com_Transmit", 1)
	SPK_COM2_ON()
end

-- ## MH Instruments ###############

-- With the exception of the Radar Altimeter, the rotaries all respond to the setting of a SHIFT
-- key, the state of which is maintained in a global variable. We can set and clear
-- this variable by calling the ON/OFF functions, typically from the down/up events of a button.
-- This makes most sense when it is the central push-button of a rotary control. Note that unshifted
-- operation is coarse control, shifted is fine control.
--
-- This functionality is  possible in principle  using LINDA's local or global shift functions, but
-- those functions appear to be problematic in practice.

local BUG_MULTIPLIER = 10						-- How fast to do coarse adjustments
local RotaryShiftActive = 0

function Rotary_Shift_ON()
	RotaryShiftActive = 1
end

function Rotary_Shift_OFF()
	RotaryShiftActive = 0
end

function ADF_Card_Knob_Left()

	local nSteps = 1
	
	if RotaryShiftActive == 1 then
		nSteps = BUG_MULTIPLIER
	end

	for n = 1, nSteps do
		_ADF_CARD_DEC()
	end

	end

function ADF_Card_Knob_Right()

	local nSteps = 1

	if RotaryShiftActive == 1 then
		nSteps = BUG_MULTIPLIER
	end

	for n = 1, nSteps do
		_ADF_CARD_INC()
	end

end

function VOR1_OBS_Left()

	local nSteps = 1
	
	if RotaryShiftActive == 1 then
		nSteps = BUG_MULTIPLIER
	end

	for n = 1, nSteps do
		_VOR1_OBI_DEC()
	end

end

function VOR1_OBS_Right()

	local nSteps = 1
	
	if RotaryShiftActive == 1 then
		nSteps = BUG_MULTIPLIER
	end

	for n = 1, nSteps do
		_VOR1_OBI_INC()
	end

end

function VOR2_OBS_Left()

	local nSteps = 1
	
	if RotaryShiftActive == 1 then
		nSteps = BUG_MULTIPLIER
	end

	for n = 1, nSteps do
		_VOR2_OBI_DEC()
	end

end

function VOR2_OBS_Right()

	local nSteps = 1
	
	if RotaryShiftActive == 1 then
		nSteps = BUG_MULTIPLIER
	end

	for n = 1, nSteps do
		_VOR2_OBI_INC()
	end

end

function Heading_Bug_Left()

	local nSteps = 1
	
	if RotaryShiftActive == 1 then
		nSteps = BUG_MULTIPLIER
	end

	for n = 1, nSteps do
		_HEADING_BUG_DEC()
	end

end

function Heading_Bug_Right()

	local nSteps = 1
	
	if RotaryShiftActive == 1 then
		nSteps = BUG_MULTIPLIER
	end

	for n = 1, nSteps do
		_HEADING_BUG_INC()
	end

end

local FEET_PER_METRE = 3.2808

function Radar_Altimeter_DH_Knob_Left()

	-- The scale isn't linear so we want to change in larger
	-- increments beyond 500ft. One call adjusts by 10ft.
	
	nCurrentDH = Round(ipc.readDBL(0x3428) * FEET_PER_METRE)
	
	if nCurrentDH <= 500 then
		_DECREASE_DECISION_HEIGHT()
	else
		_DECREASE_DECISION_HEIGHT()
		_DECREASE_DECISION_HEIGHT()
		_DECREASE_DECISION_HEIGHT()
		_DECREASE_DECISION_HEIGHT()
		_DECREASE_DECISION_HEIGHT()
	end
end

function Radar_Altimeter_DH_Knob_Right()

	-- The scale isn't linear so we want to change in larger
	-- increments beyond 500ft. One call adjusts by 10ft.

	nCurrentDH = Round(ipc.readDBL(0x3428) * FEET_PER_METRE)

	if nCurrentDH < 500 then
		_INCREASE_DECISION_HEIGHT()
	else
		_INCREASE_DECISION_HEIGHT()
		_INCREASE_DECISION_HEIGHT()
		_INCREASE_DECISION_HEIGHT()
		_INCREASE_DECISION_HEIGHT()
		_INCREASE_DECISION_HEIGHT()
	end
end

function Altimeter_Knob_Left()

	local nSteps = 1
	
	if RotaryShiftActive == 1 then
		nSteps = BUG_MULTIPLIER
	end

	for n = 1, nSteps do
		_KOHLSMAN_DEC()
	end

end

function Altimeter_Knob_Right()

	local nSteps = 1
	
	if RotaryShiftActive == 1 then
		nSteps = BUG_MULTIPLIER
	end

	for n = 1, nSteps do
		_KOHLSMAN_INC()
	end

end

-- ## MH Altitude Alerter ###############

function AA_Knob_Left()
	if RotaryShiftActive == 1 then
		TW_ALT_minusfast()
	else
		TW_ALT_minus()
	end
end

function AA_Knob_Right()
	if RotaryShiftActive == 1 then
		TW_ALT_plusfast()
	else
		TW_ALT_plus()
	end
end

function AA_MDA()
    ipc.writeLvar("L:FDR_AP_MDA_Mode", 1)
    ipc.writeLvar("L:AP_MDA", nToggleVal(ipc.readLvar("L:AP_MDA"), 0, 1))
end

function AA_Go_Around()
	_AP_ALT_HOLD_OFF()
    ipc.writeLvar("L:AP_IAS_HOLD", 0)
    ipc.writeLvar("L:AP_GA", nToggleVal(ipc.readLvar("L:AP_GA"), 0, 1))
    ipc.writeLvar("L:FDR_AP_GA_Mode", 1)
end

function AA_Alt_Alert()
    ipc.writeLvar("L:AP_AltAlert", nToggleVal(ipc.readLvar("L:AP_AltAlert"), 0, 1))
end

-- ## MH Test Panel ###############

function Fire_Bell_Mute_ON()
	ipc.WriteLvar("L:FireBellMute_Sw", 1)
end

function Fire_Bell_Mute_OFF()
	ipc.WriteLvar("L:FireBellMute_Sw", 0)
end

function Autofeather_Test_ON()
	ipc.WriteLvar("L:Autofeather_Test_Guard", 1)
	ipc.WriteLvar("L:Autofeather_Test_Sw", 1)
end

function Autofeather_Test_OFF()
	ipc.WriteLvar("L:Autofeather_Test_Sw", 0)
	ipc.WriteLvar("L:Autofeather_Test_Guard", 0)
end

function Prop_Governor_Test_ON()
	ipc.WriteLvar("L:PropOverspeed_Test_Guard", 1)
	ipc.WriteLvar("L:PropOverspeed_Test_Sw", 1)
end

function Prop_Governor_Test_OFF()
	ipc.WriteLvar("L:PropOverspeed_Test_Guard", 0)
	ipc.WriteLvar("L:PropOverspeed_Test_Sw", 0)
end

function Beta_Range_Test_ON()
	ipc.WriteLvar("L:BetaRange_Test_Sw", 1)
end

function Beta_Range_Test_OFF()
	ipc.WriteLvar("L:BetaRange_Test_Sw", 0)
end

function Power_Lever_Test_ON()
	ipc.WriteLvar("L:PowerLev_Test_PTT", 1)
end

function Power_Lever_Test_OFF()
	ipc.WriteLvar("L:PowerLev_Test_PTT", 0)
end

-- ## MH Fire ###############

function FIRE_Pull_Left_ON ()
    ipc.writeLvar("L:Firehandle_L", 1)
    ipc.writeLvar("L:Firehandle_L_anim", 1)
end

function FIRE_Pull_Left_OFF ()
    ipc.writeLvar("L:Firehandle_L", 0)
    ipc.writeLvar("L:Firehandle_L_anim", 0)
end

function FIRE_Pull_Right_ON ()
    ipc.writeLvar("L:Firehandle_R", 1)
    ipc.writeLvar("L:Firehandle_R_anim", 1)
end

function FIRE_Pull_Right_OFF ()
    ipc.writeLvar("L:Firehandle_R", 0)
    ipc.writeLvar("L:Firehandle_R_anim", 0)
end

-- ## MH Radios ###############

function DME_Mode_RMT()
    ipc.writeLvar("L:Dme_mode", 0)
end

function DME_Mode_FREQ()
    ipc.writeLvar("L:Dme_mode", 1)
end

function DME_Mode_GST()
    ipc.writeLvar("L:Dme_mode", 2)
end

function ADF_ADF()
    ipc.writeLvar("L:Mod_ADF_ADF", nToggleVal(ipc.readLvar("L:Mod_ADF_ADF"), 0, 1))

	ipc.WriteLvar("L:Mod_ADF_ADF_but", 1)
	ipc.sleep(100)
	ipc.WriteLvar("L:Mod_ADF_ADF_but", 0)
end

function ADF_BFO()
    ipc.writeLvar("L:Mod_ADF_BFO", nToggleVal(ipc.readLvar("L:Mod_ADF_BFO"), 0, 1))
	
	ipc.WriteLvar("L:Mod_ADF_BFO_but", 1)
	ipc.sleep(100)
	ipc.WriteLvar("L:Mod_ADF_BFO_but", 0)

end

function ADF_FRQ()
	ipc.WriteLvar("L:Mod_ADF_FRQ", 1)
	ipc.WriteLvar("L:Mod_ADF_FRQ_but", 1)
	ipc.sleep(100)
	ipc.WriteLvar("L:Mod_ADF_FRQ", 0)
	ipc.WriteLvar("L:Mod_ADF_FRQ_but", 0)

end

function ADF_FLT_ET()
	
	ipc.WriteLvar("L:Mod_ADF_FLT", nToggleVal(ipc.ReadLvar("L:Mod_ADF_FLT"), 1, 2))
	
	ipc.WriteLvar("L:Mod_ADF_FLT_but", 1)
	ipc.sleep(100)
	ipc.WriteLvar("L:Mod_ADF_FLT_but", 0)

end

function ADF_SET_RST_Down()
	-- We have separate down/up functions because holding down the
	-- button for 1+ seconds causes a switch to count-down mode.
	
	ipc.WriteLvar("L:Mod_ADF_SET_but", 1)
	ipc.WriteLvar("L:Mod_ADF_SET", 1)
end

function ADF_SET_RST_Up()
	ipc.WriteLvar("L:Mod_ADF_SET_but", 0)
	ipc.WriteLvar("L:Mod_ADF_SET", 0)
end

-- ADF dual concentric knobs, functions by Finn Jacobsen & Mark Hurst

local SAITEK_ADF_ROTARY_FIX = 0	-- Set this to 1 if you are using the dual concentric knob
								-- on a Saitek radio panel to be your ADF tuning knob.

-- If the above flag is set to 1, the ADF rotaries use the following 'too fast'
-- function in an attempt to mitigate the nasty rotary controls on the Saitek radio
-- panel, which I (MDH) was originally using for the ADF. We basically ignore any click
-- that arrives too soon after the previous click, thereby eliminating the Saitek's spurious
-- extra clicks. The interval value used here is a practical minimum determined through
-- experiments.
--
-- The penalty for this approach is that we can't make fast changes with these
-- controls. However, by holding down any button that is bound to the rotary SHIFT
-- function we can suspend the 'too fast' fix and make faster changes. The Saitek dual
-- concentric itself doesn't have a central push-button but we can hold down any other rotary
-- control to make use of its shift function (assuming its push-button calls the Rotary_Shift_ON()
-- and Rotary_Shift_OFF() functions).

local ADFKnobLastTime = ipc.elapsedtime()
local SAITEK_INTERVAL = 310	-- Milliseconds

local function ADFTooFast()

	if SAITEK_ADF_ROTARY_FIX == 0 then return false end		-- No speed limit 'cos we ain't using Saiteks. (Yippee!)
	if RotaryShiftActive == 1 then return false end			-- No speed limit 'cos we have a shift key held down.

	-- Otherwise go ahead and check we haven't been called again to quickly
	
	local TimeNow = ipc.elapsedtime()
	local Interval = (TimeNow - ADFKnobLastTime)
	ADFKnobLastTime = TimeNow
	--ipc.log("Interval = " .. Interval)
	
	return Interval < SAITEK_INTERVAL
end

function ADF_Outer_Knob_Left()

	if ADFTooFast() then return end

    local ADFMode = ipc.readLvar("Mod_ADF_Mode")
	
	if ADFMode == 4 then
		-- Setting the count-down timer, decrementing the
		-- minutes. 0 - 59 allowed.
		local Minutes = ipc.readLvar("ADF_min") - 1

		if Minutes < 0 then
			Minutes = 59
		end
		ipc.writeLvar("L:ADF_min", Minutes)
	else
		-- Setting the frequency, decrementing the upper
		-- digits. 01 - 17 allowed.
		local FreqHiDigits = ipc.readLvar("ADF_SSss") - 1

		if FreqHiDigits < 1 then
			FreqHiDigits = 17
		end
		ipc.writeLvar("L:ADF_SSss", FreqHiDigits)
    end

    DspShow ("ADF Outer", "Decr")

end

function ADF_Outer_Knob_Right()

	if ADFTooFast() then return end

    local ADFMode = ipc.readLvar("Mod_ADF_Mode")
	
	if ADFMode == 4 then
		-- Setting the count-down timer, incrementing the
		-- minutes. 0 - 59 allowed.
		local Minutes = ipc.readLvar("ADF_min") + 1

		if Minutes > 59 then
			Minutes = 0
		end
		ipc.writeLvar("L:ADF_min", Minutes)
	else
		-- Setting the frequency, incrementing the upper
		-- digits. 01 - 17 allowed.
		local FreqHiDigits = ipc.readLvar("ADF_SSss") + 1

		if FreqHiDigits > 17 then
			FreqHiDigits = 1
		end
		ipc.writeLvar("L:ADF_SSss", FreqHiDigits)
    end

    DspShow ("ADF Outer", "Incr")

end

function ADF_Inner_Knob_Left()

	if ADFTooFast() then return end

    local ADFMode = ipc.readLvar("Mod_ADF_Mode")
	
	if ADFMode == 4 then
		-- Setting the count-down timer, decrementing the
		-- seconds. 0 - 59 allowed.
		local Seconds = ipc.readLvar("ADF_sec") - 1

		if Seconds < 0 then
			Seconds = 59
		end
		ipc.writeLvar("L:ADF_sec", Seconds)
	else
		-- Setting the frequency, decrementing the lower
		-- digits. 00 - 99 allowed.
		local FreqLoDigits = ipc.readLvar("ADF_ttTT") - 1

		if FreqLoDigits < 0 then
			FreqLoDigits = 99
		end
		ipc.writeLvar("L:ADF_ttTT", FreqLoDigits)
    end

    DspShow ("ADF Inner", "Decr")

end

function ADF_Inner_Knob_Right()

	if ADFTooFast() then return end

    local ADFMode = ipc.readLvar("Mod_ADF_Mode")
	
	if ADFMode == 4 then
		-- Setting the count-down timer, incrementing the
		-- seconds. 0 - 59 allowed.
		local Seconds = ipc.readLvar("ADF_sec") + 1

		if Seconds > 59 then
			Seconds = 0
		end
		ipc.writeLvar("L:ADF_sec", Seconds)
	else
		-- Setting the frequency, incrementing the lower
		-- digits. 00 - 99 allowed.
		local FreqLoDigits = ipc.readLvar("ADF_ttTT") + 1

		if FreqLoDigits > 99 then
			FreqLoDigits = 0
		end
		ipc.writeLvar("L:ADF_ttTT", FreqLoDigits)
    end

    DspShow ("ADF Inner", "Incr")

end

-- ## MH Fuel ###############

function Fuel_Selector_Both_Aft()
    ipc.WriteLvar("L:Fuel_selector", -1)
end

function Fuel_Selector_Normal()
    ipc.WriteLvar("L:Fuel_selector", 0)
end

function Fuel_Selector_Both_Fwd()
    ipc.WriteLvar("L:Fuel_selector", 1)
end

function Fuel_Pump_Wing_L_ON()
    ipc.WriteLvar("L:FuelWingtank_L", 1)
end

function Fuel_Pump_Wing_L_OFF()
    ipc.WriteLvar("L:FuelWingtank_L", 0)
end

function Fuel_Pump_Wing_R_ON()
    ipc.WriteLvar("L:FuelWingtank_R", 1)
end

function Fuel_Pump_Wing_R_OFF()
    ipc.WriteLvar("L:FuelWingtank_R", 0)
end

function Fuel_Emergency_Switch_Left_Normal()
	ipc.WriteLvar("L:FuelEmerg_L", 1)
end

function Fuel_Emergency_Switch_Left_OFF()
	ipc.WriteLvar("L:FuelEmerg_L", 0)
end

function Fuel_Emergency_Switch_Right_Normal()
	ipc.WriteLvar("L:FuelEmerg_R", 1)
end

function Fuel_Emergency_Switch_Right_OFF()
	ipc.WriteLvar("L:FuelEmerg_R", 0)
end

-- ## MH Misc ###############

function Elevator_Trim_Down()
	_ELEV_TRIM_DN()
end

function Elevator_Trim_Up()
	_ELEV_TRIM_UP()
end

function Aileron_Trim_Left()
	_AILERON_TRIM_LEFT()
end

function Aileron_Trim_Right()
	_AILERON_TRIM_RIGHT()
end

function Aileron_Trim_Centre()
	_AILERON_TRIM_SET()
end

function Radar_Altimeter_Test_ON()
    ipc.WriteLvar("L:Radar_alt_Testbut", 1)
end

function Radar_Altimeter_Test_OFF()
    ipc.WriteLvar("L:Radar_alt_Testbut", 0)
end

function Caution_Light_Test_ON()
    ipc.WriteLvar("L:Ann_Test", 1)
end

function Caution_Light_Test_OFF()
    ipc.WriteLvar("L:Ann_Test", 0)
end

function Auto_Feather_Arm_Toggle()
	_TOGGLE_AUTOFEATHER_ARM()
end

function AP_TestBut_ON ()
    ipc.writeLvar("L:AP_Test bool", 1)
end

function AP_TestBut_OFF ()
    ipc.writeLvar("L:AP_Test bool", 0)
end

function DC_Load_Indicator_Left_GEN()
    ipc.writeLvar("L:Ampmeter_switch", -1)
end

function DC_Load_Indicator_Batt()
    ipc.writeLvar("L:Ampmeter_switch", 0)
end

function DC_Load_Indicator_Right_GEN()
    ipc.writeLvar("L:Ampmeter_switch", 1)
end

function Armrest_Up()
    ipc.writeLvar("L:Twot_Armrest", 1)
end

function Armrest_Down()
    ipc.writeLvar("L:Twot_Armrest", 0)
end

function Armrest_Toggle()
    ipc.writeLvar(nToggleVal(ipc.ReadLvar("L:Twot_Armrest"), 0, 1))
end

function Control_Lock_ON()
    ipc.writeLvar("L:Twot_Gustlock", 1)
	Armrest_Down()
end

function Control_Lock_OFF()
    ipc.writeLvar("L:Twot_Gustlock", 0)
	Armrest_Up()
end

function Control_Lock_Toggle()
    ipc.writeLvar(nToggleVal(ipc.ReadLvar("L:Twot_Gustlock"), 0, 1))
end

function Flight_Data_Recorder_ON()
    ipc.writeLvar("L:FDR_Main", 1)
end

function Flight_Data_Recorder_OFF()
    ipc.writeLvar("L:FDR_Main", 0)
end

function Flight_Data_Recorder_Test_DOWN()
    ipc.writeLvar("L:FDR_Test", 1)
    ipc.writeLvar("L:FDR_Event", 1)
end

function Flight_Data_Recorder_Test_UP()
	ipc.writeLvar("L:FDR_Test", 0)
    ipc.writeLvar("L:FDR_Event", 0)
end

function Feather_Left_Prop()
	_PROP_PITCH1_HI()
	_PROP_PITCH1_DECR()
end

function Unfeather_Left_Prop()
	_PROP_PITCH1_HI()
end

function Feather_Right_Prop()
	_PROP_PITCH2_HI()
	_PROP_PITCH2_DECR()
end

function Unfeather_Right_Prop()
	_PROP_PITCH2_HI()
end

function Static_Source_Normal()
    ipc.writeLvar("L:Pitotstatic", 0)
end

function Static_Source_Emergency()
    ipc.writeLvar("L:Pitotstatic", 1)
end

function Gear_Down()
	_GEAR_DOWN()
end

function Gear_Up()
	_GEAR_UP()
end